/*
 * File: Action.h
 *
 * Description: Action data holder
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef ACTION_H_
#define ACTION_H_

#include <vector>

class ParseTree;
class AgentBodyStates;

class Action
{
    private:
        struct State
        {
                State() :
                    state(-1), starter(false)
                {
                }

                int state;
                bool starter;
        };

    public:
        enum ActType
        {
            RUNONCE,
            CONTINUOUS
        };

    public:
        Action();
        virtual ~Action();

        void LoadAction(ParseTree *action, AgentBodyStates &agentStates);
        int NextState() const;
        bool ChangeState();

        void Stop(bool stopActing = true)
        {
            stop = stopActing;
        }

        int FirstState() const
        {
            return actStates[activeSection][0].state;
        }

        void Lock()
        {
            locked = true;
        }

        void Reset()
        {
            stop = false;
            activeStateNo = 0;
            if (actStates[0].empty())
                activeSection = 1;
            else
                activeSection = 0;
        }

    private:
        typedef std::vector<State> StateVector;

        bool haveStarter;
        ActType runType;
        unsigned activeSection;
        unsigned activeStateNo;
        bool stop;
        bool locked;
        StateVector actStates[3];

        void LoadStates(int statesType, ParseTree *parser,
                AgentBodyStates &agentStates);
};

#endif /*ACTION_H_*/
