/*
 * File: BasePlayer.cpp
 *
 * Description: Implementation for player's basic skills
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "BasePlayer.h"
#include "LogLevels.h"
#include "Exception.h"
#include <cmath>
#include <iostream>
#include <functional>

using namespace std;

ostream &operator<<( ostream &os, AgentBody::JointInfo joint )
{
    os << joint.Axis(AgentBody::AX1) << '\t' << joint.Axis(AgentBody::AX2);
    return os;
}

BasePlayer::BasePlayer()
{
#ifdef EMBED_DATA
    LoadDataFile("");
#else
    LoadFilesFromList("data/datafiles.list");
#endif
//    SetActionType(WALK, "walk");
    SetActionType(RESET, "reset");
}

BasePlayer::~BasePlayer()
{
}

bool BasePlayer::MoveTo(const PolarVector3D &target)
{
    Log(LOG_ACTION) << "MoveTo " << target;
    // TODO: Implement move to algorithm!
    return false;
}

bool BasePlayer::Turn(AngDeg angle)
{
    // TODO: Implement turning
    return false;
}

void BasePlayer::Walk()
{
    Log(LOG_ACTION) << "Walking";
    // TODO: implement walking
//    SetAction(WALK);
}

void BasePlayer::SideWalk(direction dir)
{
    // TODO: implement sidewalk skill
}

bool BasePlayer::Kick()
{
    // TODO: implement it!
    return false;
}

void BasePlayer::GetUp()
{
    Log(LOG_ACTION)<<"GetUp";
    // TODO: implement it
}

AngDeg BasePlayer::WhatAngleToTurnToPos( const Vector3D &target)
{
    PolarVector3D Relative = PolarVector3D( target - wm.MyPos() );
    AngDeg R = gNormalizeDeg(  /*-wm.XYBodyDirection() + */Relative.Theta() );
    return R;
}

void BasePlayer::TestAct()
{
    Vector3D gyro = wm.GyroRate();
    AgentBody::ForceResistanceInfo leftFoot = wm.FootTouchForceInfo(AgentBody::LEFT_FOOT);
    AgentBody::ForceResistanceInfo rightFoot = wm.FootTouchForceInfo(AgentBody::RIGHT_FOOT);
    Log(LOG_TEST) << "Gyro: " << gyro << "\nleft FRP: center->" << leftFoot.center
                    << " force->" << leftFoot.force << "\nright FRP: center->" << rightFoot.center
                    << " force->" << rightFoot.force;
    AgentBody::JointInfo left = wm.JointInformation(AgentBody::LEFT_LEG_MIDDLE);
    AgentBody::JointInfo right = wm.JointInformation(AgentBody::RIGHT_LEG_MIDDLE);
    Log(LOG_TEST) << "Left Rates: (" << left.Rate(AgentBody::AX1) << ' ' << left.Rate(AgentBody::AX2)
                << ")\nRight Rates: (" << right.Rate(AgentBody::AX1) << ' ' << right.Rate(AgentBody::AX2) << ')';

    float rate = max(-0.6, left.Rate(AgentBody::AX1)-0.01);
    if (left.Axis(AgentBody::AX1) < -6)
        rate = min(0.0 ,left.Rate(AgentBody::AX1)+0.1);
    SetJointVelocity(AgentBody::RIGHT_LEG_MIDDLE, rate);
    SetJointVelocity(AgentBody::LEFT_LEG_MIDDLE, rate);
}
