/*
 * File: Exception.h
 *
 * Description: The base exception class and helper functions
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef EXCEPTION_H_
#define EXCEPTION_H_

#include <string>
#include <exception>

using namespace std;

class Exception: public std::exception
{
    public:
        Exception( string m = "An Unknown Exception Occurred" ): msg( m ) {}

        const char *what() const throw()
        {
            return msg.c_str();
        }

        ~Exception() throw() {}

    protected:
        string msg;
};

inline void ThrowIf( bool condition, string msg )
{
    if ( condition )
        throw Exception( msg );
}

template<typename ExceptionType>
inline void ThrowIf( bool condition )
{
    if ( condition )
        throw ExceptionType();
}

template<typename ExceptionType>
inline void ThrowIf( bool condition, string msg )
{
    if ( condition )
        throw ExceptionType( msg );
}

#endif /*EXCEPTION_H_*/
