/*
 * File: ParseTree.cpp
 *
 * Description: ParseTree class implementation
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2006
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ctype.h>
#include <fstream>
#include "Exception.h"
#include "ParseTree.h"
#include "LogLevels.h"
using namespace std;

#ifdef EMBED_DATA
#include <sstream>
#include "agent.data.h"
#endif

void ParseTree::Parse( const string &message, string::size_type &pos )
{
    try
    {
        //cerr << message << endl << endl << endl;
        while ( pos < message.size() && message[pos] != ')' )
        {
            bool first = false;
            if ( message[pos] == '(' )
            {
                first = true;
                pos++;
            }

            string key;
            for ( ; !isspace( message.at( pos ) ) && message[pos] != ')' ; pos++ )
                key += message[pos];
            while ( isspace( message[pos] ) )
                pos++;

            if ( message.at( pos ) == '(' || ( first && message[pos] != ')' ) )
            {
                ParseTreePtr ptree( new ParseTree( message, pos ) );
                if (!ptree->IsOk())
                {
                    isOk = false;
                    return;
                }
                data.push_back( make_pair( key, ptree ) );
            }
            else
                data.push_back( make_pair( key, ParseTreePtr() ) );

            while ( isblank(message[pos]) )
                pos++;
        }
        pos++;
    }
    catch (exception &e)
    {
        isOk = false;
        Log(LOG_ERROR) << "Parsing failed: " << e.what();
    }
    catch (...)
    {
        isOk = false;
    }
}

string ParseTree::LoadFileToString(const char *filename)
{
#ifdef EMBED_DATA
    stringstream stateFile;
    // load from buffer
#else
    ifstream stateFile(filename);
#endif
    if (!stateFile)
    {
        Log(LOG_ERROR) << "ParseTree loader: cannot open file " << filename;
        return "";
    }

    string line, fileContents;
    while (stateFile)
    {
        getline(stateFile, line);

        // removing file comments
        string::size_type commentPos = line.find('/');
        if (commentPos != string::npos)
            line.resize(commentPos);

        fileContents += line + ' ';
    }

    return fileContents;
}

void ParseTree::Show(string indent)
{
    for (unsigned i = 0; i < GetNumOfKeys(); ++i)
    {
        Log(LOG_DEBUG) << indent << i << "th Key is: " << GetKey(i);
        if (GetData(i))
            GetData(i)->Show(indent + "  ");
    }
}
