/*
 * File: Player.cpp
 *
 * Description: Player class implementation
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Player.h"
#include "LogLevels.h"

Player::Player(const string teamName): act(PLAY)
{
    wm.SetTeamName(teamName);
}

Player::~Player()
{
}

void Player::SetActMode( ActionMode act )
{
    this->act =  act;
}

void Player::SetTeamName( const string teamName )
{
    wm.SetTeamName(teamName);
}

void Player::Think()
{
    static bool firtCycle = true;
    switch(act)
    {
        case PLAY:
            Play();
            break;
        case CUSTOM:
            if (firtCycle)
                Beam(-5,0,0);
            // TODO: Run a custom command
            break;
        case PENALTY:
            if (wm.PlayMode() == BEFORE_KICKOFF)
            {
                Beam(-11.0, 0, 0);
                return;
            }
            Kick();
            break;
        case NOTHING:
            break;
    }
    firtCycle = false;
}


inline void Player::Play()
{
    const float zrotate = 0;
    static Vector3D myHome;
    Log(LOG_ACTION)<<"time : "<<wm.GameTime();
    Log(LOG_ACTION)<<"............................................\n";

    switch (wm.PlayMode()) {
        case BEFORE_KICKOFF:
            switch (wm.PlayerNumber()) {
                case 1://for goalie
                    myHome.Set(-24, 0, 0);
                    break;
                case 2:
                    myHome.Set(-4.5, -0.0, 0);
                    break;
                default:
                    myHome.Set(-4.5, -0.0, 0);
                    break;
            }
            SetAction(RESET);
            Log(LOG_ACTION)<<"Reset!";
            Beam(myHome.x(), myHome.y(), zrotate);

            break;
        case KICKOFF_THEM:
        case KICKIN_THEM:
        case CORNER_KICK_THEM:
        case GOAL_KICK_THEM:
        case FREE_KICK_THEM:
        case GOAL_THEM:
            Beam(myHome.x(), myHome.y(), zrotate);
            if( wm.IsInGround() )
                GetUp();
            break;
        default:
            Log(LOG_ACTION)<<"Unknown playMode: "<<wm.PlayMode() << ", assuming play on!!!";
        case KICKIN_US:
        case PLAYON:
        case KICKOFF_US:
            if( wm.IsInGround() )
                GetUp();
            else if (wm.PlayerNumber() == 1)
            {
                Log(LOG_ACTION)<<"go to play_goalie()*******";
                // TODO: goalie!
//                Play_Goalie();
            }
            else //if(HaveBall())
            {
                Vector2D goalpos =
                        (Vector2D(wm.GetGlobalPosition(GOAL_RIGHT_THEM) )
                                + wm.GetGlobalPosition(GOAL_LEFT_THEM) )
                                / 2.0;
                Vector2D ballpos = wm.GetGlobalPosition(BALL);
                Vector2D goalToBall = ballpos - goalpos;
                goalToBall.SetLength(1);
                PolarVector3D kickPos = wm.GetRelativePolarVectorTo(ballpos + goalToBall);
                float dist = kickPos.Length2D();
                Log(LOG_ACTION) << "ball dist: " << dist;
                if (!MoveTo(kickPos))
                {
                    PolarVector3D toGoal = wm.GetGlobalRelativePolarVectorTo(goalpos);
                    float thetadiff = gNormalizeDeg(toGoal.Theta()
                            - PolarVector3D(Vector3D(-goalToBall)).Theta());
                    Log(LOG_ACTION) << "Kick pos reached, diff angle: " << thetadiff;
                    if (!Turn(thetadiff))
                    {
                        Log(LOG_ACTION) << "Kicing....!";
                        Kick();
                    }
                }
            }
        break;
    }
}

void Player::TestPlay()
{
    Log(LOG_ACTION)<<"time : "<<wm.GameTime();
    Log(LOG_ACTION)<<"............................................\n";
    static bool init = false;

    if (!init)
    {
        SetAction(RESET);
        Beam(-10, 0, 0);
        init = true;
    }
//    else if( wm.IsInGround() )
//        GetUp();
    else
        TestAct();
    Log(LOG_ACTION)<<"............................................\n";
}
