/*
 * File: SExpressionWriter.cpp
 *
 * Description: A simple class to write data in S-Expression
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "SExpressionWriter.h"

const string SExpressionWriter::INDENT = "   ";

SExpressionWriter &SExpressionWriter::BeginSub(string name)
{
    indent++;
    if (lastOutput != END_SUB)
    {
        output << '\n';
        Indent();
    }
    else
        output << INDENT;
    output << '(' << name;
    lastOutput = BEGIN_SUB;
    return *this;
}

SExpressionWriter &SExpressionWriter::EndSub()
{
    output << ")\n";
    indent--;
    Indent();
    lastOutput = END_SUB;
    return *this;
}

SExpressionWriter &SExpressionWriter::AddVector3D(string name, const Vector3D &vec)
{
    BeginSub(name);
    Add(vec.x());
    Add(vec.y());
    Add(vec.z());
    EndSub();
    return *this;
}

SExpressionWriter &SExpressionWriter::AddPolarVector(string name, const PolarVector3D &vec)
{
    BeginSub(name);
    Add(vec.Length());
    Add(vec.Theta());
    Add(vec.Phi());
    EndSub();
    return *this;
}

inline void SExpressionWriter::Indent()
{
    for (int i = 0; i < indent; ++i)
        output << INDENT;
}
