/*
 * File: SExpressionWriter.h
 *
 * Description: S-Expression writer class implementation
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SEXPRESSIONWRITER_H_
#define SEXPRESSIONWRITER_H_

#include <ostream>
#include <string>
#include "Vector.h"
#include "PolarVector.h"
using namespace std;

/**
 * This is a simple class to write data as S-Expression. All of the functions
 * return a reference to *this so that they can be concated.
 * @author Hedayat Vatankhah
 */
class SExpressionWriter
{
    private:
        enum Output
        {
            BEGIN_SUB,
            VALUE,
            END_SUB
        };

    public:
        SExpressionWriter(ostream &os, string baseName) :
            output(os), finished(false), indent(0), lastOutput(BEGIN_SUB)
        {
            output << '(' << baseName;
        }
        virtual ~SExpressionWriter()
        {
            Finish();
        }

        SExpressionWriter &BeginSub(string name);
        SExpressionWriter &EndSub();
        SExpressionWriter &AddVector3D(string name, const Vector3D &vec);
        SExpressionWriter &AddPolarVector(string name, const PolarVector3D &vec);

        template <class T>
        SExpressionWriter &Add(const T &val)
        {
            output << ' ' << val;
            lastOutput = VALUE;
            return *this;
        }

        SExpressionWriter &Endl()
        {
            output << '\n';
            return *this;
        }

        void Finish()
        {
            if (!finished)
            {
                while (indent > 0)
                    EndSub();
                output << ")\n\n";
                finished = true;
            }
        }

    private:
        const static string INDENT;
        ostream &output;
        bool finished;
        int indent;
        Output lastOutput;

        void Indent();
};

#endif /*SEXPRESSIONWRITER_H_*/
