/*
 * File: TransitionManager.cpp
 *
 * Description: TransitionManager class implementation
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef PARSER_SAFE
#define PARSER_SAFE
#endif

#include "TransitionManager.h"
#include "ParseTree.h"
#include "AgentBodyStates.h"
#include "LogLevels.h"

TransitionManager::TransitionManager(AgentBodyStates &states) :
    states(states)
{
}

TransitionManager::~TransitionManager()
{
}

void TransitionManager::LoadTransition(ParseTree *transition)
{
    Log(LOG_TRANSITION_DATA) << "\nLoading Transition...";
    int sourceState = -1, destState = -1;

    for (unsigned i = 0; i < transition->GetNumOfKeys() &&
            (sourceState == -1 || destState == -1); ++i)
    {
        if (transition->GetKey(i) == "From")
        {
//			Log(LOG_TRANSITION_DATA) << "\nFrom: " << transition->GetData(i)->GetKey(0);
            sourceState = states.GetStateNumber(transition->GetData(i)->GetKey(0));
        }
        else if (transition->GetKey(i) == "To")
            destState = states.GetStateNumber(transition->GetData(i)->GetKey(0));
    }

    Log(LOG_TRANSITION_DATA) << "Loading transition from state "<< sourceState
            << " to "<< destState;

    transitions[make_pair(sourceState, destState)].ReadTransitionData(transition);
}
