/*
 * File: TransitionManager.h
 *
 * Description: Manages Transition Data Objects
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef TRANSITIONMANAGER_H_
#define TRANSITIONMANAGER_H_

#include <map>
#include "TransitionData.h"
#include "AgentBodyStates.h"
using namespace std;

class ParseTree;

class TransitionManager
{
    public:
        TransitionManager(AgentBodyStates &states);
        virtual ~TransitionManager();

        void LoadTransition(ParseTree *transition);

        TransitionData *Transition(int sourceState, int destState)
        {
            TransitionMap::iterator
                    trptr = transitions.find(make_pair(sourceState, destState));
            if (trptr==transitions.end())
            {
                Log(LOG_ERROR) << "Transition not available from "
                        << states.StateName(sourceState) << " to "
                        << states.StateName(destState);
                return NULL;
            }
            return &trptr->second;
        }

        void SetTransitionParameter(int sourcest, int destst, const char *name, float val)
        {
            TransitionData *transition = Transition(sourcest, destst);
            if (transition)
                transition->SetParameter(name, val);
        }

    private:
        typedef map<pair<int, int>, TransitionData > TransitionMap;

        TransitionMap transitions;
        AgentBodyStates &states;
};

#endif /*TRANSITIONMANAGER_H_*/
