/*
 * File: WorldModelCore.inline.h
 *
 * Description: WorldModelCore inline functions
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef WORLDMODELCORE_INLINE_H_
#define WORLDMODELCORE_INLINE_H_

#ifdef NO_INLINE
#define inline
#endif

inline bool WorldModelCore::IsActive() const
{
    return running;
}

inline string WorldModelCore::TeamName() const
{
    return teamName;
}

inline float WorldModelCore::Time() const
{
    return simtime;
}

inline float WorldModelCore::GameTime() const
{
    return gameTime;
}

inline AngDeg WorldModelCore::JointAxis(AgentBody::Joint joint, AgentBody::JointAxis ax) const
{
    return agent.JointData(joint).Axis(ax);
}

inline AgentBody::JointInfo WorldModelCore::JointInformation(AgentBody::Joint joint) const
{
    return agent.JointData(joint);
}

inline float WorldModelCore::GetSpentTime(float start) const
{
    return simtime - start;
}

inline Vector3D WorldModelCore::MyPos() const
{
    return MyPosition;
}

inline Vector3D WorldModelCore::BallVel() const
{
    return BallVelocity;
}

inline PlayModeType WorldModelCore::PlayMode() const
{
    return playMode;
}

inline int WorldModelCore::PlayerNumber() const
{
    return playerNumber;
}

inline bool WorldModelCore::IsInGround() const
{
    return MyPosition.z() < 0.5/* ||
           (!FootTouchForceInfo(AgentBody::RIGHT_FOOT).IsAny() &&
                   !FootTouchForceInfo(AgentBody::LEFT_FOOT).IsAny())*/;
}

inline const PolarVector3D &WorldModelCore::GetRelativePolarVectorTo(VisionObject obj) const
{
    return visionObjectsPolar[obj];
}

inline PolarVector3D WorldModelCore::GetRelativePolarVectorTo(const Vector3D &globalPosition) const
{
    const Vector3D &rel = xInvRotation.Rotate(yInvRotation.Rotate(zInvRotation.Rotate(globalPosition-MyPosition)));
    return zRotation.InverseRotate(yRotation.InverseRotate(xRotation.InverseRotate(rel)));
}

inline void WorldModelCore::Stop()
{
    running = false;
}

inline bool WorldModelCore::IsNewSenseInfo()
{
    bool tmp = newSenseInfo;
    newSenseInfo = false;
    return tmp;
}

inline void WorldModelCore::SetTeamName(const string &teamName)
{
    this->teamName = teamName;
}

inline const Vector3D &WorldModelCore::GyroRate() const
{
    return agent.GyroRate();
}

inline const AgentBody::ForceResistanceInfo &WorldModelCore::FootTouchForceInfo(
        AgentBody::AgentFoot foot) const
{
    return agent.FootTouchForceInfo(foot);
}

inline void WorldModelCore::SaveAgentState(ostream &os, const string &stateName)
{
    agent.SaveAgentState(os, stateName);
}

inline float WorldModelCore::FieldWidth() const
{
    return 32;
}

inline float WorldModelCore::FieldLength() const
{
    return 50;
}

inline float WorldModelCore::GoalWidth() const
{
    return 7.32;
}

inline Vector3D WorldModelCore::GetGlobalPosition(VisionObject obj) const
{
    if (Types::IsFlag(obj))
        return flagPosition[obj];
    else
        return MyPosition + Rotate(GetRelativePolarVectorTo(obj));
}

inline PolarVector3D WorldModelCore::GetGlobalRelativePolarVectorTo(VisionObject obj) const
{
    return Rotate(GetRelativePolarVectorTo(obj));
}

inline PolarVector3D WorldModelCore::GetGlobalRelativePolarVectorTo(const Vector3D &globalPosition) const
{
    return globalPosition - MyPosition;
}

inline Vector3D WorldModelCore::Rotate(const Vector3D &relative) const
{
    const Vector3D &rel = xRotation.Rotate(yRotation.Rotate(zRotation.Rotate(relative)));
    return zInvRotation.InverseRotate(yInvRotation.InverseRotate(xInvRotation.InverseRotate(rel)));
}

#endif /*WORLDMODELCORE_INLINE_H_*/
