/*
 * File: main.cpp
 *
 * Description: Agent's start up function (main)
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <fstream>
#include <string>
#include <ncurses.h>
#include "Player.h"
#include "LogLevels.h"
#include "CursesOStreamBuffer.h"
#include "rcssnet/tcpsocket.hpp"

using namespace std;
using namespace rcss::net;

bool CheckOptions(int ac, char *av[], Player &player, Addr::PortType &port, string &host,
        bool &userCommand, string &logFileName);

void SetupLogger()
{
// 	Log.addLogLevel(LOG_DEBUG);
//    Log.addLogLevel(LOG_DEBUG_SENSE);
//	Log.addLogLevel(LOG_DEBUG_STREADER);
    Log.addLogLevel(LOG_WARNING);
    Log.addLogLevel(LOG_ERROR);
//	Log.addLogLevel(LOG_HIGHLEVEL);
//    Log.addLogLevel(LOG_ACTION);
 	Log.addLogLevel(LOG_CONNECTION);
// 	Log.addLogLevel(LOG_WALK);
//    Log.addLogLevel(LOG_ACTIONPARSER);
    Log.addLogLevel(LOG_ACTION_MANAGER);
//	Log.addLogLevel(LOG_TRANSITION_DATA);
// 	Log.addLogLevel(LOG_VISION);
//	Log.addLogLevel(LOG_JOINT_CONTROLLER);
//	Log.addLogLevel(LOG_FRPCHECK);
//    Log.addLogLevel(LOG_TEST);
//	Log.addLogLevel(LOG_HRH_WHICH);
}

int main(int argc, char *argv[])
{
    cout << "UI-AI 3D 2008 Spark Agent.\n"
         << "Copyright (c) 2008, UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>.\n"
         << "University of Isfahan.\n" << endl;

    Addr::PortType port = 3100;
    string host = "127.0.0.1";
    bool userCommand = false;
    CursesOStreamBuffer cursesOSBuf;
    ostream cursesOS(&cursesOSBuf);
    ofstream logFile;
    string logFileName;

    try
    {
        SetupLogger();
        Player player("UI-AI");
        if (CheckOptions(argc, argv, player, port, host, userCommand, logFileName))
        {
            if (userCommand)
            {
                // Initializing ncurses window
                scrollok(initscr(), true);
                raw();
                keypad(stdscr, TRUE);
                noecho();
                Log.setOutputStream(cursesOS);
            }

            if (!logFileName.empty())
            {
                logFile.open(logFileName.c_str());
                ThrowIf(!logFile, string("Cannot open log file: ") + logFileName);
                Log.setOutputStream(logFile);
            }
            player.Connect(Addr(port, host));
//			player.SetAgentType(BasePlayer::OLD_AGENT);
#ifdef FINAL_RELEASE
            // No user command through stdin
            player.Run();
#else
            player.Run(userCommand);
#endif
        }
    }
    catch (exception &e)
    {
        Log(LOG_ERROR) << "Oh, No! -> " << e.what();
        if (userCommand)
            getch();
    }

    if (userCommand)
    {
        refresh();
        endwin();
    }
}

void PrintHelp()
{
    cout << "\nUsage: uiai [OPTOINS]\nOptions:"
            "\n\t--help: show this screen"
#ifndef FINAL_RELEASE
            "\n\t--teamname <teamname>: set the team name of the player"
#endif
            "\n\t--host <hostname>: set host name of the server"
            "\n\t--port <port_no>: set the port number to connect to"
            "\n\t--logfile <filename>: set log file name"
#ifndef FINAL_RELEASE
            "\n\t--penalty: penalty kick"
            "\n\t--nothing: don't do anything! it is useful with --user-input"
            "\n\t--load <filename>: load data file"
            "\n\t--user-input: enable agent control through stdin"
            "\n\t--custom <filename>: run the custom action file"
#endif
            "\n\n";
}

bool CheckOptions(int ac, char *av[], Player &player, Addr::PortType &port, string &host,
        bool &userCommand, string &logFileName)
{
    for (int i = 1; i < ac; ++i)
    {
        if (strcmp(av[i], "--help") == 0)
        {
            PrintHelp();
            return false;
        }
        else if (strcmp(av[i], "--user-input") == 0)
            userCommand = true;
#ifndef FINAL_RELEASE
        else if (strcmp(av[i], "--teamname") == 0)
            player.SetTeamName(av[++i]);
#endif
        else if (strcmp(av[i], "--host") == 0)
            host = av[++i];
        else if (strcmp(av[i], "--port") == 0)
            port = atoi(av[++i]);
        else if (strcmp(av[i], "--load") == 0)
            player.LoadDataFile(av[++i]);
        else if (strcmp(av[i], "--penalty") == 0)
            player.SetActMode(Player::PENALTY);
        else if (strcmp(av[i], "--nothing") == 0)
            player.SetActMode(Player::NOTHING);
        else if (strcmp(av[i], "--logfile") == 0)
            logFileName = av[++i];
        else if (strcmp(av[i], "--custom") == 0)
        {
            player.SetActMode(Player::CUSTOM);
        }
        else
            cout << "What?! \"" << av[i] << "\"?!! ;)" << endl;
    }
    return true;
}
